@@ -34,6 +34,12 @@
 #  include <config.h>
 #endif
 
+#include <fnmatch.h>
+
+#ifndef FNM_CASEFOLD
+#  define FNM_CASEFOLD 0
+#endif
+
 #include <gnome.h>
 
 #include "gsearchtool.h"
@@ -43,7 +49,6 @@
 #include <string.h>
 #include <unistd.h>
 #include <signal.h>
-#include <fnmatch.h>
 #include <libbonobo.h>
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
@@ -60,21 +65,21 @@ struct _FindOptionTemplate {
 };
 	
 static FindOptionTemplate templates[] = {
-	{ SEARCH_CONSTRAINT_TEXT, "-exec grep -c '%s' {} \\;", N_("Contains the text"), FALSE },
+	{ SEARCH_CONSTRAINT_TEXT, "-exec grep -c '%s' {} \\;", N_("C_ontains the text"), FALSE },
 	{ SEARCH_CONSTRAINT_SEPARATOR, NULL, NULL, TRUE },
-	{ SEARCH_CONSTRAINT_TIME, "-mtime -%d", N_("Date modified less than (days)"), FALSE },
-	{ SEARCH_CONSTRAINT_TIME, "-mtime +%d", N_("Date modified more than (days)"), FALSE },
+	{ SEARCH_CONSTRAINT_TIME, "-mtime -%d", N_("_Date modified less than (days)"), FALSE },
+	{ SEARCH_CONSTRAINT_TIME, "-mtime +%d", N_("Da_te modified more than (days)"), FALSE },
 	{ SEARCH_CONSTRAINT_SEPARATOR, NULL, NULL, TRUE },
-	{ SEARCH_CONSTRAINT_NUMBER, "-size +%uc", N_("Size at least (kilobytes)"), FALSE }, 
-	{ SEARCH_CONSTRAINT_NUMBER, "-size -%uc", N_("Size at most (kilobytes)"), FALSE },
+	{ SEARCH_CONSTRAINT_NUMBER, "-size +%uc", N_("_Size at least (kilobytes)"), FALSE }, 
+	{ SEARCH_CONSTRAINT_NUMBER, "-size -%uc", N_("Si_ze at most (kilobytes)"), FALSE },
 	{ SEARCH_CONSTRAINT_BOOL, "-size 0c \\( -type f -o -type d \\)", N_("File is empty"), FALSE },	
 	{ SEARCH_CONSTRAINT_SEPARATOR, NULL, NULL, TRUE },
-	{ SEARCH_CONSTRAINT_TEXT, "-user '%s'", N_("Owned by user"), FALSE },
-	{ SEARCH_CONSTRAINT_TEXT, "-group '%s'", N_("Owned by group"), FALSE },
+	{ SEARCH_CONSTRAINT_TEXT, "-user '%s'", N_("Owned by _user"), FALSE },
+	{ SEARCH_CONSTRAINT_TEXT, "-group '%s'", N_("Owned by _group"), FALSE },
 	{ SEARCH_CONSTRAINT_BOOL, "\\( -nouser -o -nogroup \\)", N_("Owner is unrecognized"), FALSE },
 	{ SEARCH_CONSTRAINT_SEPARATOR, NULL, NULL, TRUE },
-	{ SEARCH_CONSTRAINT_TEXT, "'!' -name '%s'", N_("File is not named"), FALSE },
-	{ SEARCH_CONSTRAINT_TEXT, "-regex '%s'", N_("File matches regular expression"), FALSE }, 
+	{ SEARCH_CONSTRAINT_TEXT, "'!' -name '*%s*'", N_("Na_me does not contain"), FALSE },
+	{ SEARCH_CONSTRAINT_TEXT, "-regex '%s'", N_("Name matches regular e_xpression"), FALSE }, 
 	{ SEARCH_CONSTRAINT_SEPARATOR, NULL, NULL, TRUE },
 	{ SEARCH_CONSTRAINT_BOOL, "-follow", N_("Follow symbolic links"), FALSE },
 	{ SEARCH_CONSTRAINT_BOOL, "-xdev", N_("Include other filesystems"), FALSE },
@@ -107,7 +112,8 @@ enum {
 	SEARCH_CONSTRAINT_FILE_MATCHES_REGULAR_EXPRESSION,
 	SEARCH_CONSTRAINT_SEPARATOR_04,	
 	SEARCH_CONSTRAINT_FOLLOW_SYMBOLIC_LINKS,
-	SEARCH_CONSTRAINT_SEARCH_OTHER_FILESYSTEMS
+	SEARCH_CONSTRAINT_SEARCH_OTHER_FILESYSTEMS,
+	SEARCH_CONSTRAINT_MAXIMUM_POSSIBLE
 };
 
 struct _PoptArgument {	
@@ -153,6 +159,53 @@ struct poptOption options[] = {
   	{ NULL,'\0', 0, NULL, 0, NULL, NULL}
 };
 
+static gchar *find_command_default_name_option;
+static gchar *locate_command_default_options;
+	
+void   
+setup_case_insensitive_arguments (void)
+{
+	static gboolean first_pass = TRUE;
+	gchar *cmd_stderr;
+
+	if (first_pass != TRUE) {
+		return;
+	}
+
+	first_pass = FALSE;
+
+	/* check find command for -iname argument compatibility */
+	g_spawn_command_line_sync ("find /dev/null -name 'string'", NULL, &cmd_stderr, NULL, NULL);
+
+	if ((cmd_stderr != NULL) && (strlen (cmd_stderr) == 0)) {
+		find_command_default_name_option = g_strdup ("-iname");
+		templates[SEARCH_CONSTRAINT_FILE_IS_NOT_NAMED].option = g_strdup ("'!' -iname '*%s*'");
+	}
+	else {
+		find_command_default_name_option = g_strdup ("-name");
+	}
+	g_free (cmd_stderr);
+
+	/* check grep command for -i argument compatibility */
+	g_spawn_command_line_sync ("grep -i 'string' /dev/null", NULL, &cmd_stderr, NULL, NULL);
+
+	if ((cmd_stderr != NULL) && (strlen (cmd_stderr) == 0)) {
+		templates[SEARCH_CONSTRAINT_CONTAINS_THE_TEXT].option = g_strdup ("-exec grep -i -c '%s' {} \\;");
+	}
+	g_free (cmd_stderr);
+
+	/* check locate command for -i argument compatibility */
+	g_spawn_command_line_sync ("locate -i /dev/null/string", NULL, &cmd_stderr, NULL, NULL);
+
+	if ((cmd_stderr != NULL) && (strlen (cmd_stderr) == 0)) {
+		locate_command_default_options = g_strdup ("-i");
+	}
+	else {
+		locate_command_default_options = g_strdup ("");
+	}
+	g_free (cmd_stderr);
+}
+
 gchar *
 build_search_command (void)
 {
@@ -162,6 +215,8 @@ build_search_command (void)
 	gchar *file_is_named_escaped;
 	gchar *look_in_folder_utf8;
 	gchar *look_in_folder_locale;
+
+	setup_case_insensitive_arguments ();
 	
 	file_is_named_utf8 = (gchar *) gtk_entry_get_text (GTK_ENTRY(gnome_entry_gtk_entry (GNOME_ENTRY(interface.file_is_named_entry))));
 
@@ -169,7 +224,16 @@ build_search_command (void)
 		file_is_named_utf8 = g_strdup ("*");
 	} 
 	else {
+		gchar *locale;
+
+		locale = g_locale_from_utf8 (file_is_named_utf8, -1, NULL, NULL, NULL);
 		gnome_entry_prepend_history (GNOME_ENTRY(interface.file_is_named_entry), TRUE, file_is_named_utf8);
+
+		if (strstr (locale, "*") == NULL) {
+			file_is_named_utf8 = g_strconcat ("*", file_is_named_utf8, "*", NULL);
+		}
+		
+		g_free (locale);
 	}
 	file_is_named_locale = g_locale_from_utf8 (file_is_named_utf8, -1, NULL, NULL, NULL);
 	
@@ -193,16 +257,18 @@ build_search_command (void)
 		file_is_named_escaped = escape_single_quotes (file_is_named_locale);
 		search_command.file_is_named_pattern = g_strdup(file_is_named_utf8);
 		
-		if ((locate != NULL) && (is_path_in_home_folder (look_in_folder_locale) != TRUE)
-		    && (is_path_in_mount_folder (look_in_folder_locale) != TRUE)) {
-			g_string_append_printf (command, "%s '%s*%s'", 
-						locate, 
+		if ((locate != NULL) && (is_path_in_home_folder (look_in_folder_locale) != TRUE) 
+		     && (is_path_in_mount_folder (look_in_folder_locale) != TRUE)) {	
+			g_string_append_printf (command, "%s %s '%s*%s'", 
+						locate,
+						locate_command_default_options,
 						look_in_folder_locale,
 						file_is_named_escaped);
 		} 
 		else {
-			g_string_append_printf (command, "find \"%s\" '!' -type p -name '%s' -xdev -print", 
+			g_string_append_printf (command, "find \"%s\" '!' -type p %s '%s' -xdev -print", 
 						look_in_folder_locale, 
+						find_command_default_name_option, 
 						file_is_named_escaped);
 		}
 		g_free (locate);
@@ -215,16 +281,10 @@ build_search_command (void)
 		file_is_named_escaped = escape_single_quotes (file_is_named_locale);
 		search_command.file_is_named_pattern = g_strdup(file_is_named_utf8);
 		
-		if (file_is_named_escaped == NULL) {
-			g_string_append_printf (command, "find \"%s\" '!' -type p -name '%s' ", 
-					 	look_in_folder_locale,
-						"*");
-		}
-		else {
-			g_string_append_printf (command, "find \"%s\" '!' -type p -name '%s' ", 
+		g_string_append_printf (command, "find \"%s\" '!' -type p %s '%s' ", 
 					 	look_in_folder_locale,
+						find_command_default_name_option,
 						file_is_named_escaped);
-		}
 	
 		for (list = interface.selected_constraints; list != NULL; list = g_list_next (list)) {
 			
@@ -261,13 +321,16 @@ build_search_command (void)
 					string = escape_single_quotes (constraint->data.text);
 					locale = g_locale_from_utf8 (string, -1, NULL, NULL, NULL);
 					
-					g_string_append_printf (command,
-						  	        templates[constraint->constraint_id].option,
-						  	        locale);
-					g_free(string);
-					g_free(locale);
+					if (strlen (locale) != 0) { 
+						g_string_append_printf (command,
+							  	        templates[constraint->constraint_id].option,
+						  		        locale);
 
-					g_string_append_c (command, ' ');
+						g_string_append_c (command, ' ');
+					}
+					
+					g_free(string);
+					g_free(locale);					
 				}
 				break;
 			case SEARCH_CONSTRAINT_NUMBER:
@@ -428,14 +491,15 @@ make_list_of_templates (void)
 			gtk_widget_show (menuitem);
 		} 
 		else {
-			menuitem = gtk_radio_menu_item_new_with_label (group,
-							      	_(templates[i].desc));
+			gchar *text = remove_mnemonic_character (templates[i].desc);
+			menuitem = gtk_radio_menu_item_new_with_label (group, _(text));
 			g_signal_connect (G_OBJECT(menuitem), "toggled",
 					  G_CALLBACK(constraint_menu_toggled_cb),
 		        		  (gpointer)(long)i);
 			group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (menuitem));
 			gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
 			gtk_widget_show (menuitem);
+			g_free (text);
 		}
 		
 		if (templates[i].is_selected == TRUE) {
@@ -508,6 +572,69 @@ set_constraint_selected_state (gint 		constraint_id,
 	gtk_widget_set_sensitive (interface.add_button, FALSE);
 }
 
+void
+set_constraint_gconf_boolean (gint constraint_id, gboolean flag)
+{
+	switch (constraint_id) {
+	
+		case SEARCH_CONSTRAINT_CONTAINS_THE_TEXT:
+			gsearchtool_gconf_set_boolean ("/apps/gnome-search-tool/contains_the_text",
+	   		       	       	       	       flag);
+			break;
+		case SEARCH_CONSTRAINT_DATE_MODIFIED_BEFORE:
+			gsearchtool_gconf_set_boolean ("/apps/gnome-search-tool/date_modified_less_than",
+	   		       	       	       	       flag);
+			break;
+		case SEARCH_CONSTRAINT_DATE_MODIFIED_AFTER:
+			gsearchtool_gconf_set_boolean ("/apps/gnome-search-tool/date_modified_more_than",
+	   		       	       	       	       flag);
+			break;
+		case SEARCH_CONSTRAINT_SIZE_IS_MORE_THAN:
+			gsearchtool_gconf_set_boolean ("/apps/gnome-search-tool/size_at_least",
+	   		       	       		       flag);
+			break;
+		case SEARCH_CONSTRAINT_SIZE_IS_LESS_THAN:
+			gsearchtool_gconf_set_boolean ("/apps/gnome-search-tool/size_at_most",
+	   		       	       	  	       flag);
+			break;
+		case SEARCH_CONSTRAINT_FILE_IS_EMPTY:
+			gsearchtool_gconf_set_boolean ("/apps/gnome-search-tool/file_is_empty",
+	   		       	       	               flag);
+			break;
+		case SEARCH_CONSTRAINT_OWNED_BY_USER:
+			gsearchtool_gconf_set_boolean ("/apps/gnome-search-tool/owned_by_user",
+	   		       	       	               flag);
+			break;
+		case SEARCH_CONSTRAINT_OWNED_BY_GROUP:
+			gsearchtool_gconf_set_boolean ("/apps/gnome-search-tool/owned_by_group",
+	   		       	       	               flag);
+			break;
+		case SEARCH_CONSTRAINT_OWNER_IS_UNRECOGNIZED:
+			gsearchtool_gconf_set_boolean ("/apps/gnome-search-tool/owner_is_unrecognized",
+	   		       	       	               flag);
+			break;
+		case SEARCH_CONSTRAINT_FILE_IS_NOT_NAMED:
+			gsearchtool_gconf_set_boolean ("/apps/gnome-search-tool/name_does_not_contain",
+	   		       	       	               flag);
+			break;
+		case SEARCH_CONSTRAINT_FILE_MATCHES_REGULAR_EXPRESSION:
+			gsearchtool_gconf_set_boolean ("/apps/gnome-search-tool/name_matches_regular_expression",
+	   		       	       	               flag);
+			break;
+		case SEARCH_CONSTRAINT_FOLLOW_SYMBOLIC_LINKS:
+			gsearchtool_gconf_set_boolean ("/apps/gnome-search-tool/follow_symbolic_links",
+	   		       	       	               flag);
+			break;
+		case SEARCH_CONSTRAINT_SEARCH_OTHER_FILESYSTEMS:
+			gsearchtool_gconf_set_boolean ("/apps/gnome-search-tool/include_other_filesystems",
+	   		       	       	               flag);
+			break;
+		
+		default:
+			break;
+	}
+}
+
 static void
 setup_app_progress_bar (void)
 {
@@ -590,82 +717,101 @@ define_popt_descriptions (void)
 	gint i = 0;
 	gint j;
 
-	options[i++].descrip = g_strdup (_("Set the text of 'file is named'"));
-	options[i++].descrip = g_strdup (_("Set the text of 'look in folder'"));
+	options[i++].descrip = g_strdup (_("Set the text of 'Name contains'"));
+	options[i++].descrip = g_strdup (_("Set the text of 'Look in folder'"));
   	options[i++].descrip = g_strdup (_("Sort files by one of the following: name, folder, size, type, or date"));
   	options[i++].descrip = g_strdup (_("Set sort order to descending, the default is ascending"));
   	options[i++].descrip = g_strdup (_("Automatically start a search"));
 
 	for (j = 0; templates[j].type != SEARCH_CONSTRAINT_END; j++) {
 		if (templates[j].type != SEARCH_CONSTRAINT_SEPARATOR) {
-			options[i++].descrip = g_strdup_printf (_("Select the '%s' constraint"), templates[j].desc);
+			gchar *text = remove_mnemonic_character (templates[j].desc);
+			options[i++].descrip = g_strdup_printf (_("Select the '%s' constraint"), _(text));
+			g_free (text);
 		}
 	}
 }
 
-void 
+static gboolean
 handle_popt_args (void)
 {
+	gboolean popt_args_found = FALSE;
 	gint sort_by;
 
 	if (PoptArgument.name != NULL) {
+		popt_args_found = TRUE;
 		gtk_entry_set_text (GTK_ENTRY(gnome_entry_gtk_entry (GNOME_ENTRY(interface.file_is_named_entry))),
 				    g_locale_to_utf8 (PoptArgument.name, -1, NULL, NULL, NULL));
 	}
 	if (PoptArgument.path != NULL) {
+		popt_args_found = TRUE;
 		gtk_entry_set_text (GTK_ENTRY(gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY(interface.look_in_folder_entry))),
 				    g_locale_to_utf8 (PoptArgument.path, -1, NULL, NULL, NULL));
 	}	
 	if (PoptArgument.contains != NULL) {
+		popt_args_found = TRUE;	
 		add_constraint (SEARCH_CONSTRAINT_CONTAINS_THE_TEXT, 
-				PoptArgument.contains); 
-	}
-	if (PoptArgument.user != NULL) {
-		add_constraint (SEARCH_CONSTRAINT_OWNED_BY_USER, 
-				PoptArgument.user); 
-	}
-	if (PoptArgument.group != NULL) {
-		add_constraint (SEARCH_CONSTRAINT_OWNED_BY_GROUP, 
-				PoptArgument.group); 
-	}
-	if (PoptArgument.nouser == TRUE) {
-		add_constraint (SEARCH_CONSTRAINT_OWNER_IS_UNRECOGNIZED, NULL); 
+				PoptArgument.contains, TRUE); 	
 	}
 	if (PoptArgument.mtimeless != NULL) {
+		popt_args_found = TRUE;
 		add_constraint (SEARCH_CONSTRAINT_DATE_MODIFIED_BEFORE, 
-				PoptArgument.mtimeless); 
+				PoptArgument.mtimeless, TRUE);		
 	}
 	if (PoptArgument.mtimemore != NULL) {
+		popt_args_found = TRUE;
 		add_constraint (SEARCH_CONSTRAINT_DATE_MODIFIED_AFTER, 
-				PoptArgument.mtimemore); 
-	}
-	if (PoptArgument.sizeless != NULL) {
-		add_constraint (SEARCH_CONSTRAINT_SIZE_IS_LESS_THAN,
-				PoptArgument.sizeless);
+				PoptArgument.mtimemore, TRUE);
 	}
 	if (PoptArgument.sizemore != NULL) {
+		popt_args_found = TRUE;
 		add_constraint (SEARCH_CONSTRAINT_SIZE_IS_MORE_THAN,
-				PoptArgument.sizemore);
+				PoptArgument.sizemore, TRUE);
+	}
+	if (PoptArgument.sizeless != NULL) {
+		popt_args_found = TRUE;
+		add_constraint (SEARCH_CONSTRAINT_SIZE_IS_LESS_THAN,
+				PoptArgument.sizeless, TRUE);
 	}
 	if (PoptArgument.empty == TRUE) {
-		add_constraint (SEARCH_CONSTRAINT_FILE_IS_EMPTY, NULL);
+		popt_args_found = TRUE;
+		add_constraint (SEARCH_CONSTRAINT_FILE_IS_EMPTY, NULL, TRUE);
+	}
+	if (PoptArgument.user != NULL) {
+		popt_args_found = TRUE;
+		add_constraint (SEARCH_CONSTRAINT_OWNED_BY_USER, 
+				PoptArgument.user, TRUE); 
+	}
+	if (PoptArgument.group != NULL) {
+		popt_args_found = TRUE;
+		add_constraint (SEARCH_CONSTRAINT_OWNED_BY_GROUP, 
+				PoptArgument.group, TRUE);
+	}
+	if (PoptArgument.nouser == TRUE) {
+		popt_args_found = TRUE;
+		add_constraint (SEARCH_CONSTRAINT_OWNER_IS_UNRECOGNIZED, NULL, TRUE); 
 	}
 	if (PoptArgument.notnamed != NULL) {
+		popt_args_found = TRUE;
 		add_constraint (SEARCH_CONSTRAINT_FILE_IS_NOT_NAMED, 
-				PoptArgument.notnamed); 
+				PoptArgument.notnamed, TRUE);
 	}
 	if (PoptArgument.regex != NULL) {
+		popt_args_found = TRUE;
 		add_constraint (SEARCH_CONSTRAINT_FILE_MATCHES_REGULAR_EXPRESSION, 
-				PoptArgument.regex); 
+				PoptArgument.regex, TRUE);
 	}
 	if (PoptArgument.follow == TRUE) {
-		add_constraint (SEARCH_CONSTRAINT_FOLLOW_SYMBOLIC_LINKS, NULL); 
+		popt_args_found = TRUE;
+		add_constraint (SEARCH_CONSTRAINT_FOLLOW_SYMBOLIC_LINKS, NULL, TRUE); 
 	}
 	if (PoptArgument.allmounts == TRUE) {
-		add_constraint (SEARCH_CONSTRAINT_SEARCH_OTHER_FILESYSTEMS, NULL); 
+		popt_args_found = TRUE;
+		add_constraint (SEARCH_CONSTRAINT_SEARCH_OTHER_FILESYSTEMS, NULL, TRUE);
 	}
 	if (PoptArgument.sortby != NULL) {
 	
+		popt_args_found = TRUE;
 		if (strcmp (PoptArgument.sortby, "name") == 0) {
 			sort_by = COLUMN_NAME;
 		}
@@ -696,8 +842,10 @@ handle_popt_args (void)
 		}
 	}
 	if (PoptArgument.start == TRUE) {
+		popt_args_found = TRUE;
 		click_find_cb (interface.find_button, NULL);
 	}
+	return popt_args_found;
 }
 
 static gboolean
@@ -762,10 +910,10 @@ handle_search_command_stdout_io (GIOChannel 	*ioc,
 			if (strncmp (string->str, search_data->look_in_folder, look_in_folder_string_length) == 0) { 
 			
 				if (strlen (string->str) != look_in_folder_string_length) {
-				
+			
 					filename = g_path_get_basename (utf8);
 			
-					if (fnmatch (search_data->file_is_named_pattern, filename, FNM_NOESCAPE) != FNM_NOMATCH) {
+					if (fnmatch (search_data->file_is_named_pattern, filename, FNM_NOESCAPE | FNM_CASEFOLD ) != FNM_NOMATCH) {
 						if (search_data->show_hidden_files == TRUE) {
 							if (search_data->regex_matching_enabled == FALSE) {
 								add_file_to_search_results (string->str, interface.model, &interface.iter);
@@ -822,6 +970,9 @@ handle_search_command_stdout_io (GIOChannel 	*ioc,
 		search_data->lock = FALSE;
 		search_data->running = NOT_RUNNING;
 		
+		search_data->not_running_timeout = TRUE;
+		g_timeout_add (500, not_running_timeout_cb, NULL);
+
 		total_files = gtk_tree_model_iter_n_children (GTK_TREE_MODEL(interface.model), NULL);
 
 		if (total_files == 0) {
@@ -839,7 +990,7 @@ handle_search_command_stdout_io (GIOChannel 	*ioc,
 		gnome_appbar_pop (GNOME_APPBAR (interface.status_bar));
 		gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (interface.progress_bar), 0.0);
 		gnome_appbar_push (GNOME_APPBAR (interface.status_bar), status_bar_string);
-		gtk_timeout_remove (search_data->timeout);
+		g_source_remove (search_data->timeout);
 
 		gtk_widget_set_sensitive (interface.additional_constraints, TRUE);
 		gtk_widget_set_sensitive (interface.disclosure, TRUE);
@@ -975,7 +1126,7 @@ spawn_search_command (gchar *command)
 		gnome_appbar_pop (GNOME_APPBAR (interface.status_bar));
 		gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (interface.progress_bar), 0.0);
 		gnome_appbar_push (GNOME_APPBAR (interface.status_bar), "");
-		gtk_timeout_remove (search_command.timeout);
+		g_source_remove (search_command.timeout);
 		
 		dialog = gtk_message_dialog_new(GTK_WINDOW (interface.main_window), 
 						GTK_DIALOG_DESTROY_WITH_PARENT, 
@@ -999,7 +1150,7 @@ spawn_search_command (gchar *command)
 		gnome_appbar_pop (GNOME_APPBAR (interface.status_bar));
 		gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (interface.progress_bar), 0.0);
 		gnome_appbar_push (GNOME_APPBAR (interface.status_bar), "");
-		gtk_timeout_remove (search_command.timeout);
+		g_source_remove (search_command.timeout);
 		
 		dialog = gtk_message_dialog_new(GTK_WINDOW (interface.main_window), 
 						GTK_DIALOG_DESTROY_WITH_PARENT, 
@@ -1060,10 +1211,12 @@ create_constraint_box (SearchConstraint *opt, gchar *value)
 	switch(templates[opt->constraint_id].type) {
 	case SEARCH_CONSTRAINT_BOOL:
 		{
-			gchar *desc = g_strconcat (LEFT_LABEL_SPACING, _(templates[opt->constraint_id].desc), ".", NULL);
+			gchar *text = remove_mnemonic_character (templates[opt->constraint_id].desc);
+			gchar *desc = g_strconcat (LEFT_LABEL_SPACING, _(text), ".", NULL);
 			label = gtk_label_new (desc);
-			g_free (desc);
 			gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, GNOME_PAD_SMALL);
+			g_free (desc);
+			g_free (text);
 		}
 		break;
 	case SEARCH_CONSTRAINT_TEXT:
@@ -1071,7 +1224,7 @@ create_constraint_box (SearchConstraint *opt, gchar *value)
 	case SEARCH_CONSTRAINT_TIME:
 		{
 			gchar *desc = g_strconcat (LEFT_LABEL_SPACING, _(templates[opt->constraint_id].desc), ":", NULL);
-			label = gtk_label_new (desc);
+			label = gtk_label_new_with_mnemonic (desc);
 			g_free (desc);
 		}
 		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, GNOME_PAD_SMALL);
@@ -1080,12 +1233,15 @@ create_constraint_box (SearchConstraint *opt, gchar *value)
 			entry = gtk_entry_new();
 			if (value != NULL) {
 				gtk_entry_set_text (GTK_ENTRY(entry), value);
+				opt->data.text = g_strdup (value);
 			}
 		}
 		else {
 			entry = gtk_spin_button_new_with_range (0, 999999999, 1);
 			if (value != NULL) {
 				gtk_spin_button_set_value (GTK_SPIN_BUTTON(entry), atoi(value));
+				opt->data.time = atoi(value);
+				opt->data.number = atoi(value);
 			}
 		}
 		
@@ -1097,6 +1253,8 @@ create_constraint_box (SearchConstraint *opt, gchar *value)
 			add_atk_relation (GTK_WIDGET(label), entry, ATK_RELATION_LABEL_FOR);
 		}
 		
+		gtk_label_set_mnemonic_widget (GTK_LABEL (label), GTK_WIDGET (entry));
+		
 		g_signal_connect (G_OBJECT(entry), "changed",
 			 	  G_CALLBACK(constraint_update_info_cb), opt);
 		
@@ -1126,24 +1284,27 @@ create_constraint_box (SearchConstraint *opt, gchar *value)
 	gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 0);
 	
 	if (interface.is_gail_loaded) {
-		gchar *desc = g_strdup_printf (_("Click to Remove the '%s' Rule"), 
-					       _(templates[opt->constraint_id].desc));
+		gchar *text = remove_mnemonic_character (templates[opt->constraint_id].desc);
+		gchar *desc = g_strdup_printf (_("Click to Remove the '%s' Rule"), _(text));
 		add_atk_namedesc (GTK_WIDGET(button), NULL, desc);
 		g_free (desc);
+		g_free (text);
 	}
 	return hbox;
 }
 
 void
-add_constraint (gint constraint_id, gchar *value)
+add_constraint (gint constraint_id, gchar *value, gboolean show_constraint)
 {
 	SearchConstraint *constraint = g_new(SearchConstraint,1);
 	GtkWidget *widget;
 	
-	if (GTK_WIDGET_VISIBLE (interface.additional_constraints) == FALSE) {
-		g_signal_emit_by_name (G_OBJECT(interface.disclosure), "clicked", 0);
-		gtk_widget_show (interface.additional_constraints);
-	}
+	if (show_constraint == TRUE) {
+		if (GTK_WIDGET_VISIBLE (interface.additional_constraints) == FALSE) {
+			g_signal_emit_by_name (G_OBJECT(interface.disclosure), "clicked", 0);
+			gtk_widget_show (interface.additional_constraints);
+		}
+	} 
 	
 	interface.geometry.min_height += 30; 
 	gtk_window_set_geometry_hints (GTK_WINDOW(interface.main_window), 
@@ -1152,6 +1313,7 @@ add_constraint (gint constraint_id, gchar *value)
 	
 	constraint->constraint_id = constraint_id;
 	set_constraint_info_defaults (constraint);
+	set_constraint_gconf_boolean (constraint->constraint_id, TRUE);
 	
 	widget = create_constraint_box (constraint, value);
 	gtk_box_pack_start (GTK_BOX(interface.constraint), widget, FALSE, FALSE, 0);
@@ -1204,7 +1366,7 @@ create_additional_constraint_section (void)
 	gtk_size_group_add_widget (interface.constraint_size_group, interface.add_button);
 	
 	g_signal_connect (G_OBJECT(interface.add_button),"clicked",
-			  G_CALLBACK(add_constraint_cb),NULL);
+			  G_CALLBACK(add_constraint_cb), NULL);
 	
 	gtk_box_pack_end (GTK_BOX(hbox), interface.add_button, FALSE, FALSE, 0); 
 		
@@ -1220,17 +1382,17 @@ create_search_results_section (void)
 {
 	GtkWidget 	  *label;
 	GtkWidget 	  *vbox;
-	GtkWidget 	  *window;	
+	GtkWidget	  *window;	
 	GtkTreeViewColumn *column;	
 	GtkCellRenderer   *renderer;
 	
-	vbox = gtk_vbox_new (FALSE, 0);
+	vbox = gtk_vbox_new (FALSE, 0);	
 	
 	label = gtk_label_new_with_mnemonic (_("S_earch Results"));
 	g_object_set (G_OBJECT(label), "xalign", 0.0, NULL);
 	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, GNOME_PAD_SMALL);
 	
-	window = gtk_scrolled_window_new (NULL, NULL); 
+	window = gtk_scrolled_window_new (NULL, NULL);
 	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW(window), GTK_SHADOW_IN);	
 	gtk_container_set_border_width (GTK_CONTAINER(window), 0);
 	gtk_widget_set_size_request (window, 530, 160); 
@@ -1290,6 +1452,11 @@ create_search_results_section (void)
 		          G_CALLBACK(file_button_press_event_cb), 
 			  NULL);		   
 
+	g_signal_connect (G_OBJECT(interface.tree),
+			  "key_press_event",
+			  G_CALLBACK(file_key_press_event_cb),
+			  NULL);
+			  
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), interface.tree);
 	
 	gtk_container_add (GTK_CONTAINER(window), GTK_WIDGET(interface.tree));
@@ -1364,8 +1531,8 @@ create_main_window (void)
 {
 	gchar 		*string;
 	GtkWidget 	*hbox;	
-	GtkWidget 	*label;
 	GtkWidget 	*entry;
+	GtkWidget	*label;
 	GtkWidget 	*folder_entry;
 	GtkWidget 	*button;
 	GtkWidget 	*window;
@@ -1385,7 +1552,7 @@ create_main_window (void)
 	gtk_table_set_col_spacings (GTK_TABLE(interface.table), GNOME_PAD);
 	gtk_container_add (GTK_CONTAINER(hbox), interface.table);
 	
-	label = gtk_label_new_with_mnemonic (_("File is _named:"));
+	label = gtk_label_new_with_mnemonic (_("_Name contains:"));
 	gtk_label_set_justify (GTK_LABEL(label), GTK_JUSTIFY_LEFT);
 	g_object_set (G_OBJECT(label), "xalign", 0.0, NULL);
 	
@@ -1535,6 +1702,156 @@ gsearchtool_init_stock_icons ()
 	g_object_unref (factory);
 }
 
+void
+set_clone_command (gint *argcp, gchar ***argvp, gpointer client_data)
+{
+	gchar **argv;
+	gchar *file_is_named_utf8;
+	gchar *file_is_named_locale;
+	gchar *look_in_folder_utf8;
+	gchar *look_in_folder_locale;
+	GList *list;
+	gint  i = 0;
+
+	argv = g_new0 (gchar*, SEARCH_CONSTRAINT_MAXIMUM_POSSIBLE);
+
+	argv[i++] = (gchar *) client_data;
+
+	file_is_named_utf8 = (gchar *) gtk_entry_get_text (GTK_ENTRY(gnome_entry_gtk_entry (GNOME_ENTRY(interface.file_is_named_entry))));
+	file_is_named_locale = g_locale_from_utf8 (file_is_named_utf8 != NULL ? file_is_named_utf8 : "" , 
+	                                           -1, NULL, NULL, NULL);
+	argv[i++] = g_strdup_printf ("--named=%s", file_is_named_locale);
+	g_free (file_is_named_locale);
+
+	look_in_folder_utf8 = gnome_file_entry_get_full_path (GNOME_FILE_ENTRY(interface.look_in_folder_entry), FALSE);
+	look_in_folder_locale = g_locale_from_utf8 (look_in_folder_utf8 != NULL ? look_in_folder_utf8 : "", 
+	                                            -1, NULL, NULL, NULL);
+	argv[i++] = g_strdup_printf ("--path=%s", look_in_folder_locale);
+	g_free (look_in_folder_locale);
+
+	if (GTK_WIDGET_VISIBLE(interface.additional_constraints) == TRUE) {
+		for (list = interface.selected_constraints; list != NULL; list = g_list_next (list)) {
+			SearchConstraint *constraint = list->data;
+			gchar *locale = NULL;
+			
+			switch (constraint->constraint_id) {
+			case SEARCH_CONSTRAINT_CONTAINS_THE_TEXT:
+				locale = g_locale_from_utf8 (constraint->data.text, -1, NULL, NULL, NULL);
+				argv[i++] = g_strdup_printf ("--contains=%s", locale);
+				break;
+			case SEARCH_CONSTRAINT_DATE_MODIFIED_BEFORE:
+				argv[i++] = g_strdup_printf ("--mtimeless=%d", constraint->data.time);
+				break;
+			case SEARCH_CONSTRAINT_DATE_MODIFIED_AFTER:
+				argv[i++] = g_strdup_printf ("--mtimemore=%d", constraint->data.time);
+				break;
+			case SEARCH_CONSTRAINT_SIZE_IS_MORE_THAN:
+				argv[i++] = g_strdup_printf ("--sizemore=%u", constraint->data.number);
+				break;
+			case SEARCH_CONSTRAINT_SIZE_IS_LESS_THAN:
+				argv[i++] = g_strdup_printf ("--sizeless=%u", constraint->data.number);
+				break;
+			case SEARCH_CONSTRAINT_FILE_IS_EMPTY:
+				argv[i++] = g_strdup ("--empty");
+				break;
+			case SEARCH_CONSTRAINT_OWNED_BY_USER:
+				locale = g_locale_from_utf8 (constraint->data.text, -1, NULL, NULL, NULL);
+				argv[i++] = g_strdup_printf ("--user=%s", locale);
+				break;
+			case SEARCH_CONSTRAINT_OWNED_BY_GROUP:
+				locale = g_locale_from_utf8 (constraint->data.text, -1, NULL, NULL, NULL);
+				argv[i++] = g_strdup_printf ("--group=%s", locale);
+				break;
+			case SEARCH_CONSTRAINT_OWNER_IS_UNRECOGNIZED:
+				argv[i++] = g_strdup ("--nouser");
+				break;
+			case SEARCH_CONSTRAINT_FILE_IS_NOT_NAMED:
+				locale = g_locale_from_utf8 (constraint->data.text, -1, NULL, NULL, NULL);
+				argv[i++] = g_strdup_printf ("--notnamed=%s", locale);
+				break;
+			case SEARCH_CONSTRAINT_FILE_MATCHES_REGULAR_EXPRESSION:
+				locale = g_locale_from_utf8 (constraint->data.text, -1, NULL, NULL, NULL);
+				argv[i++] = g_strdup_printf ("--regex=%s", locale);
+				break;
+			case SEARCH_CONSTRAINT_FOLLOW_SYMBOLIC_LINKS:
+				argv[i++] = g_strdup ("--follow");
+				break;
+			case SEARCH_CONSTRAINT_SEARCH_OTHER_FILESYSTEMS:
+				argv[i++] = g_strdup ("--allmounts");
+				break;
+			default:
+				break;
+			}
+		g_free (locale);
+		}		
+	}
+	*argvp = argv;
+	*argcp = i;
+}
+
+void
+handle_gconf_settings (void)
+{
+	if (gsearchtool_gconf_get_boolean ("/apps/gnome-search-tool/show_additional_options") == TRUE) {
+		if (GTK_WIDGET_VISIBLE (interface.additional_constraints) == FALSE) {
+			g_signal_emit_by_name (G_OBJECT(interface.disclosure), "clicked", 0);
+			gtk_widget_show (interface.additional_constraints);
+		}
+	}
+		      
+	if (gsearchtool_gconf_get_boolean ("/apps/gnome-search-tool/contains_the_text") == TRUE) {
+		add_constraint (SEARCH_CONSTRAINT_CONTAINS_THE_TEXT, "", FALSE); 
+	}
+		
+	if (gsearchtool_gconf_get_boolean ("/apps/gnome-search-tool/date_modified_less_than") == TRUE) {
+		add_constraint (SEARCH_CONSTRAINT_DATE_MODIFIED_BEFORE, "", FALSE); 
+	}
+	
+	if (gsearchtool_gconf_get_boolean ("/apps/gnome-search-tool/date_modified_more_than") == TRUE) {
+		add_constraint (SEARCH_CONSTRAINT_DATE_MODIFIED_AFTER, "", FALSE); 
+	}
+	
+	if (gsearchtool_gconf_get_boolean ("/apps/gnome-search-tool/size_at_least") == TRUE) {
+		add_constraint (SEARCH_CONSTRAINT_SIZE_IS_MORE_THAN, "", FALSE); 
+	}
+	
+	if (gsearchtool_gconf_get_boolean ("/apps/gnome-search-tool/size_at_most") == TRUE) {
+		add_constraint (SEARCH_CONSTRAINT_SIZE_IS_LESS_THAN, "", FALSE); 
+	}
+	
+	if (gsearchtool_gconf_get_boolean ("/apps/gnome-search-tool/file_is_empty") == TRUE) {
+		add_constraint (SEARCH_CONSTRAINT_FILE_IS_EMPTY, NULL, FALSE); 
+	}
+	
+	if (gsearchtool_gconf_get_boolean ("/apps/gnome-search-tool/owned_by_user") == TRUE) {
+		add_constraint (SEARCH_CONSTRAINT_OWNED_BY_USER, "", FALSE); 
+	}
+	
+	if (gsearchtool_gconf_get_boolean ("/apps/gnome-search-tool/owned_by_group") == TRUE) {
+		add_constraint (SEARCH_CONSTRAINT_OWNED_BY_GROUP, "", FALSE); 
+	}
+	
+	if (gsearchtool_gconf_get_boolean ("/apps/gnome-search-tool/owner_is_unrecognized") == TRUE) {
+		add_constraint (SEARCH_CONSTRAINT_OWNER_IS_UNRECOGNIZED, NULL, FALSE); 
+	}
+	
+	if (gsearchtool_gconf_get_boolean ("/apps/gnome-search-tool/name_does_not_contain") == TRUE) {
+		add_constraint (SEARCH_CONSTRAINT_FILE_IS_NOT_NAMED, "", FALSE); 
+	}
+	
+	if (gsearchtool_gconf_get_boolean ("/apps/gnome-search-tool/name_matches_regular_expression") == TRUE) {
+		add_constraint (SEARCH_CONSTRAINT_FILE_MATCHES_REGULAR_EXPRESSION, "", FALSE); 
+	}
+	
+	if (gsearchtool_gconf_get_boolean ("/apps/gnome-search-tool/follow_symbolic_links") == TRUE) {
+		add_constraint (SEARCH_CONSTRAINT_FOLLOW_SYMBOLIC_LINKS, NULL, FALSE); 
+	}
+	
+	if (gsearchtool_gconf_get_boolean ("/apps/gnome-search-tool/include_other_filesystems") == TRUE) {
+		add_constraint (SEARCH_CONSTRAINT_SEARCH_OTHER_FILESYSTEMS, NULL, FALSE); 
+	}
+}
+
 int
 main (int 	argc, 
       char 	*argv[])
@@ -1543,6 +1860,7 @@ main (int 	argc,
 	GnomeClient  *client;
 	GtkWidget    *window;
 
+	global_gconf_client = NULL;
 	interface.geometry.min_height = 310;
 	interface.geometry.min_width  = 422;
 
@@ -1551,6 +1869,8 @@ main (int 	argc,
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
+	g_set_application_name (_("Search for Files"));
+
 	define_popt_descriptions ();
 
 	gsearchtool = gnome_program_init ("gnome-search-tool", VERSION, 
@@ -1608,8 +1928,10 @@ main (int 	argc,
 	
 	interface.icon_theme = gnome_icon_theme_new ();
 	
-	handle_popt_args ();
-		
+	if (handle_popt_args () == FALSE) {
+		handle_gconf_settings (); 
+	}	
+		 
 	gtk_main ();
 
 	return 0;
